package simpleanimation;

import java.awt.*;
import java.awt.geom.*;

public class FishShape implements SimpleShape{
	
	SmartEllipse body=new SmartEllipse();
	SmartArc tail=new SmartArc(90,180,Arc2D.Double.PIE);
	private int posX, posY;	
	private int width, height;
	private final int TAIL_WIDTH=12;

	Color fillColor;
	Color borderColor;
	int strokeWidth=1;
	private double rotation;
	
	public FishShape(int width, int height, Color color)
	{
		this.setSize(width, height);		
		this.setFillColor(color);
		this.setBorderColor(color);
	}
	
	public void setLocation (int posX, int posY)
	{
		this.posX=posX;
		this.posY=posY;
		body.setLocation(posX+TAIL_WIDTH/2, posY);
		tail.setLocation(posX, posY);		
	}

	public void setSize (int width, int height)
	{
		this.width=width;
		this.height=height;
		body.setSize(width-TAIL_WIDTH/2,height);
		tail.setSize(TAIL_WIDTH, height);
	}
	
	public void setFillColor(Color color)
	{
		this.fillColor=color;
		body.setFillColor(color);
		tail.setFillColor(color);
	}
	
	public void setBorderColor(Color color)
	{
		this.borderColor=color;
		body.setBorderColor(color);
		tail.setBorderColor(color);
	}
	
	public void changeLocation (int stepX, int stepY)
	{
		this.setLocation(this.posX+stepX, this.posY+stepY);
	}
		
	//method to paint the shape	
	public void fill(Graphics2D brush)
	{
		//save color in a brush to restore it
		Color savedColor=brush.getColor();
		brush.setColor(this.fillColor);
		brush.fill(this.body.shape);
		brush.fill(this.tail.shape);
		//restore the color of the brush
		brush.setColor(savedColor);		
	}
			
	public void stroke(Graphics2D brush)
	{
		//save color in a brush to restore it
		Color savedColor=brush.getColor();
		brush.setColor(this.borderColor);
		//save default stroke width to restore it
		java.awt.Stroke savedStroke=brush.getStroke();
		brush.setStroke(new java.awt.BasicStroke(this.strokeWidth));
		brush.draw(this.body.shape);
		brush.draw(this.tail.shape);
		//restore the color of the brush
		brush.setColor(savedColor);	
		//restore the default stroke width
		brush.setStroke(savedStroke);
	}		
	
	public void paint(Graphics2D brush)
	{
		if(this.rotation!=0)
		{
			brush.rotate(this.rotation, this.getCenterX(), this.getCenterY());
		}
		this.stroke(brush);
		this.fill(brush);
		if(this.rotation!=0)
		{
			brush.rotate(-this.rotation, this.getCenterX(), this.getCenterY());
		}
	}
	
	public int getX()
	{
		return this.posX;
	}
	public int getY()
	{
		return this.posY;
	}
	
	public int getWidth()
	{
		return this.width ;
	}
	public int getHeight()
	{
		return this.height;
	}
	public int getCenterX()
	{
		return this.posX+this.width/2;
	}
	public int getCenterY()
	{
		return this.posY+this.height/2;
	}
	public void setRotation(double rotation)
	{
		this.rotation=rotation;
		this.body.setRotation(rotation);
		this.tail.setRotation(rotation);
	}
	
	public double getRotation()
	{
		return this.rotation;
	}
}
