package simpleanimation;

import java.awt.*;
import java.awt.geom.*;
/**
 * The class which contains a shape which can
 * draw itself
 * rotate itself
 * move itself
 * position itself
 * color itself
 */
public class SmartRectangularShape implements SimpleShape{
	private Color  borderColor=Color.WHITE, fillColor=Color.WHITE;		
	private int strokeWidth=1;  //default stroke width 1 pixel
	private double rotation=0;
	public final RectangularShape shape;
	
	public SmartRectangularShape(RectangularShape shape)
	{
		this.shape=shape;		
	}
	
	//methods to change colors
	public void setBorderColor(Color borderColor) {
		this.borderColor = borderColor;
	}	

	public void setFillColor(Color fillColor) {
		this.fillColor = fillColor;
	}

	
	public int getCenterX()
	{
		return (int)shape.getCenterX();
	}
	
	public int getCenterY()
	{
		return (int)shape.getCenterY();
	}
	
	
	//methods to change the stroke width
	public int getStrokeWidth() {
		return strokeWidth;
	}

	public void setStrokeWidth(int strokeWidth) {
		this.strokeWidth = strokeWidth;
	}
	
	//methods for positioning the shape
	public int getX()
	{
		return (int)shape.getX();
	}
	
	public int getY()
	{
		return (int)shape.getY();
	}
	
	public void setLocation(int x, int y)
	{
		shape.setFrame(x,y,shape.getWidth(),shape.getHeight());
	}
	
	public void changeLocation(int stepX, int stepY)
	{
		this.shape.setFrame(this.getX()+stepX, this.getY()+stepY,this.getWidth(), this.getHeight());
	}
	
	//methods for shape dimensions
	public int getWidth()
	{
		return (int)shape.getWidth();
	}
	
	public int getHeight()
	{
		return (int)shape.getHeight();
	}
	
	public void setSize(int w, int h)
	{
		shape.setFrame(shape.getX(),shape.getY(),w,h);
	}
	
	//methods to paint the shape
	public void fill(Graphics2D brush)
	{
		//save color in a brush to restore it
		Color savedColor=brush.getColor();
		brush.setColor(this.fillColor);
		brush.fill(this.shape);
		//restore the color of the brush
		brush.setColor(savedColor);		
	}
		
	public void stroke(Graphics2D brush)
	{
		//save color in a brush to restore it
		Color savedColor=brush.getColor();
		brush.setColor(this.borderColor);
		//save default stroke width to restore it
		java.awt.Stroke savedStroke=brush.getStroke();
		brush.setStroke(new java.awt.BasicStroke(this.getStrokeWidth()));
		brush.draw(this.shape);
		//restore the color of the brush
		brush.setColor(savedColor);	
		//restore the default stroke width
		brush.setStroke(savedStroke);
	}
	
			
	public void paint(Graphics2D brush)
	{
		if(this.rotation!=0)
		{
			brush.rotate(this.rotation, this.getCenterX(), this.getCenterY());
		}
		this.stroke(brush);
		this.fill(brush);
		if(this.rotation!=0)
		{
			brush.rotate(-this.rotation, this.getCenterX(), this.getCenterY());
		}
	}
	
	public void setRotation(double rotation)
	{
		this.rotation=rotation;
	}
	
	public double getRotation()
	{
		return this.rotation;
	}
}
